@@ -26,7 +26,7 @@ accounts <- function(server = NULL) {
     path <- file.path(path, server)
 
   # get a raw list of accounts
-  accountnames <- tools::file_path_sans_ext(list.files(path,
+  accountnames <- file_path_sans_ext(list.files(path,
     pattern=glob2rx("*.dcf"), recursive = TRUE))
 
   if (length(accountnames) == 0) {
@@ -98,7 +98,8 @@ connectUser <- function(account = NULL, server = NULL, quiet = FALSE) {
   # keep trying to authenticate until we're successful
   repeat {
     Sys.sleep(1)
-    user <- getUserFromRawToken(target$url, token$token, token$private_key)
+    user <- getUserFromRawToken(target$url, token$token, token$private_key,
+                                target$certificate)
     if (!is.null(user))
       break
   }
@@ -164,8 +165,9 @@ setAccountInfo <- function(name, token, secret) {
     stop(stringParamErrorMessage("secret"))
 
   # create connect client
-  authInfo <- list(token = token, secret = secret)
   serverInfo <- shinyappsServerInfo()
+  authInfo <- list(token = token, secret = secret,
+                   certificate = serverInfo$certificate)
   lucid <- lucidClient(serverInfo$url, authInfo)
 
   # get user Id
@@ -253,7 +255,8 @@ getAuthToken <- function(server, userId = 0) {
 
   # generate a token and push it to the server
   token <- generateToken()
-  connect <- connectClient(service = target$url, authInfo = list())
+  connect <- connectClient(service = target$url,
+                           authInfo = list(certificate = target$certificate))
   response <- connect$addToken(list(token = token$token,
                                     public_key = token$public_key,
                                     user_id = as.integer(userId)))
@@ -268,11 +271,16 @@ getAuthToken <- function(server, userId = 0) {
 # given a server URL and raw information about an auth token, return the user
 # who owns the token, if it's claimed, and NULL if the token is unclaimed.
 # raises an error on any other HTTP error.
-getUserFromRawToken <- function(serverUrl, token, privateKey) {
+#
+# this function is used by the RStudio IDE as part of the workflow which
+# attaches a new Connect account.
+getUserFromRawToken <- function(serverUrl, token, privateKey,
+                                serverCertificate = NULL) {
   # form a temporary client from the raw token
   connect <- connectClient(service = serverUrl, authInfo =
                            list(token = token,
-                                private_key = as.character(privateKey)))
+                                private_key = as.character(privateKey),
+                                certificate = serverCertificate))
 
   # attempt to fetch the user
   user <- NULL
