@@ -174,7 +174,7 @@ bundleApp <- function(appName, appDir, appFiles, appPrimaryDoc, assetTypeName,
   # infer the mode of the application from its layout
   # unless we're an API, in which case, we're API mode.
   appMode <- inferAppMode(appDir, appPrimaryDoc, appFiles)
-  hasParameters <- appHasParameters(appDir, appFiles, contentCategory)
+  hasParameters <- appHasParameters(appDir, appFiles, appMode, contentCategory)
 
   if (verbose)
     timestampedLog("Bundling app dir")
@@ -234,11 +234,19 @@ yamlFromRmd <- function(filename) {
   return(NULL)
 }
 
-appHasParameters <- function(appDir, files, contentCategory) {
-
-  # sites don't ever have parameters
-  if (identical(contentCategory, "site"))
+appHasParameters <- function(appDir, files, appMode, contentCategory) {
+  # Only Rmd deployments are marked as having parameters. Shiny applications
+  # may distribute an Rmd alongside app.R, but that does not cause the
+  # deployment to be considered parameterized.
+  #
+  # https://github.com/rstudio/rsconnect/issues/246
+  if (!(appMode %in% c("rmd-static", "rmd-shiny"))) {
     return(FALSE)
+  }
+  # Sites don't ever have parameters
+  if (identical(contentCategory, "site")) {
+    return(FALSE)
+  }
 
   rmdFiles <- grep("^[^/\\\\]+\\.rmd$", files, ignore.case = TRUE, perl = TRUE,
                    value = TRUE)
@@ -301,6 +309,11 @@ inferAppMode <- function(appDir, appPrimaryDoc, files) {
     return("rmd-static")
   }
 
+  # We don't have an RMarkdown, Shiny app, or Plumber API, but we have a saved model
+  if(length(grep("(saved_model.pb|saved_model.pbtxt)$", files, ignore.case = TRUE, perl = TRUE)) > 0) {
+    return("tensorflow-saved-model")
+  }
+
   # no renderable content here; if there's at least one file, we can just serve
   # it as static content
   if (length(files) > 0) {
