@@ -23,7 +23,7 @@ deploySite <- function(siteDir = getwd(),
                        server = NULL,
                        render = c("none", "local", "server"),
                        launch.browser = getOption("rsconnect.launch.browser", interactive()),
-                       logLevel = c("quiet", "normal", "verbose"),
+                       logLevel = c("normal", "quiet", "verbose"),
                        lint = FALSE,
                        metadata = list()) {
 
